﻿using System;
using System.IO;
using System.IO.IsolatedStorage;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using Microsoft.Devices;
using Microsoft.Phone.Controls;
using Microsoft.Xna.Framework.Media;
using System.Diagnostics;

namespace CameraPreviewTest {
    public partial class MainPage : PhoneApplicationPage {

        // コンストラクター
        public MainPage() {
            InitializeComponent();
        }

        PhotoCamera camera = null;

        // ページがフレームでアクティブになったら呼び出される
        protected override void OnNavigatedTo(NavigationEventArgs e) {
            camera = new PhotoCamera();
            camera.Initialized += camera_Initialized;
            PreviewBrush.SetSource(camera);
        }

        // ページがフレームでアクティブでなくなったら呼び出される
        protected override void OnNavigatedFrom(NavigationEventArgs e) {
            if (camera != null) {
                camera.Dispose();
                camera = null;
            }
        }

        // プレビューフレームの解析を行うタイマー
        System.Windows.Threading.DispatcherTimer readTimer = null;

        // カメラの初期化処理の完了
        void  camera_Initialized(object sender, CameraOperationCompletedEventArgs e) {
            // 初期化処理に失敗した場合は何もしない
            if (!e.Succeeded)
                return;

            // カメラの回転角度に合わせてプレビュー表示も回転させる
            Dispatcher.BeginInvoke(() => {
                PreviewBrush.RelativeTransform = new CompositeTransform() {
                    CenterX = 0.5, CenterY = 0.5,
                    Rotation = camera.Orientation
                };

                ImageEffect.RenderTransform = new CompositeTransform() {
                    CenterX = 0.5, CenterY = 0.5,
                    Rotation = camera.Orientation
                };

                // 100ミリ秒毎に満了する様にタイマーを開始
                readTimer = new System.Windows.Threading.DispatcherTimer();
                readTimer.Tick += new EventHandler(readTimer_Tick);
                readTimer.Interval = TimeSpan.FromMilliseconds(100);
                readTimer.Start();
            });
        }

        // タイマー満了時にプレビューフレームにエフェクト処理
        void readTimer_Tick(object sender, EventArgs e) {

            var frame = new FrameBuffer(
                (int)camera.PreviewResolution.Width,
                (int)camera.PreviewResolution.Height);
            camera.GetPreviewBufferArgb32(frame.Buffer);

            frame.EffectSepiaTone();

            // 取り込んだ画像をWriteableBitmapへセット
            var wb = new WriteableBitmap(frame.Width, frame.Height);
            frame.Buffer.CopyTo(wb.Pixels, 0);
            // 
            ImageEffect.Source = wb;
        }

        // 表示領域がタップされるとオートフォーカス処理を開始する
        private void PreviewRectangle_Tap(object sender, System.Windows.Input.GestureEventArgs e) {
            if (camera == null) {
                return;
            }

            System.Diagnostics.Debug.WriteLine("AutoFocus Start: " + DateTime.Now.ToString("HH:mm:ss.fff"));
            camera.AutoFocusCompleted += camera_AutoFocusCompleted;
            // オートフォーカス処理を開始する
            camera.Focus();
        }

        // オートフォーカスの完了
        void camera_AutoFocusCompleted(object sender, CameraOperationCompletedEventArgs e) {
            camera.AutoFocusCompleted -= camera_AutoFocusCompleted;
            System.Diagnostics.Debug.WriteLine("AutoFocus End  : " + DateTime.Now.ToString("HH:mm:ss.fff"));
        }

        private void menuItemTakePicture_Click(object sender, EventArgs e) {

            // フラッシュモードの設定を行う
            // カメラが赤目補正モードに対応しているか判定する
            if (camera.IsFlashModeSupported(FlashMode.RedEyeReduction)) {
                camera.FlashMode = FlashMode.RedEyeReduction;
            }

            // 静止画撮影可能な解像度を取得する
            var sizes = camera.AvailableResolutions;
            // 撮影可能な解像度のうち一番大きなものを設定
            camera.Resolution = sizes.Last();

            // 撮影シーケンス時に発行されるイベントにハンドラを設定
            camera.CaptureStarted += new EventHandler(camera_CaptureStarted);
            camera.CaptureImageAvailable += new EventHandler<ContentReadyEventArgs>(camera_CaptureImageAvailable);
            camera.CaptureThumbnailAvailable += new EventHandler<ContentReadyEventArgs>(camera_CaptureThumbnailAvailable);
            camera.CaptureCompleted += new EventHandler<CameraOperationCompletedEventArgs>(camera_CaptureCompleted);

            // 静止画撮影の開始要求
            camera.CaptureImage();
        }

        // 静止画撮影の開始
        void camera_CaptureStarted(object sender, EventArgs e) {
            Debug.WriteLine("camera_CaptureStarted");
        }

        // カメラからの静止画取り込み終了し画像が使用可能になった
        void camera_CaptureImageAvailable(object sender, ContentReadyEventArgs e) {
            Debug.WriteLine("camera_CaptureImageAvailable");

            try {
                // 分離ストレージへキャプチャーデータへ保存する
                using (var store = IsolatedStorageFile.GetUserStoreForApplication())
                using (var strm = store.CreateFile("capture_image.jpg")) {
                    var bytes = new byte[256 * 1024];
                    while (true) {
                        int read = e.ImageStream.Read(bytes, 0, bytes.Length);
                        if (read <= 0) break;
                        strm.Write(bytes, 0, read);
                    }
                }
            } catch (Exception ex) {
                // ここでエラー処理
            } finally {
                e.ImageStream.Close();
            }
        }

        // カメラからの静止画取り込み終了しサムネイル画像が使用可能になった
        void camera_CaptureThumbnailAvailable(object sender, ContentReadyEventArgs e) {
            Debug.WriteLine("camera_CaptureThumbnailAvailable");

            try {
                // 分離ストレージへサムネイルデータへ保存する
                using (var store = IsolatedStorageFile.GetUserStoreForApplication())
                using (var strm = store.CreateFile("thumbnail.jpg")) {
                    var bytes = new byte[256 * 1024];
                    while (true) {
                        int read = e.ImageStream.Read(bytes, 0, bytes.Length);
                        if (read <= 0) break;
                        strm.Write(bytes, 0, read);
                    }
                }
            } catch (Exception ex) {
                // ここでエラー処理
            } finally {
                e.ImageStream.Close();
            }
        }

        // 静止画撮影の完了
        void camera_CaptureCompleted(object sender, CameraOperationCompletedEventArgs e) {
            Debug.WriteLine("camera_CaptureCompleted");

            if (e.Succeeded) {
                // 静止画撮影が成功した時の処理
            } else {
                // 静止画撮影が失敗した時の処理 
            }
        }
    }
}